/* ************************************************************************** */
/*                                                                            */
/*                                                        :::      ::::::::   */
/*   main.c                                             :+:      :+:    :+:   */
/*                                                    +:+ +:+         +:+     */
/*   By: mmoussou <mmoussou@student.42angouleme.fr  +#+  +:+       +#+        */
/*                                                +#+#+#+#+#+   +#+           */
/*   Created: 2024/04/24 11:18:04 by adjoly            #+#    #+#             */
/*   Updated: 2024/07/04 10:49:59 by mmoussou         ###   ########.fr       */
/*                                                                            */
/* ************************************************************************** */

#include <signal.h>
#include <stdio.h>
#include <readline/readline.h>
#include "error_msg.h"
#include <readline/history.h>
#include <stdlib.h>
#include <stdbool.h>
#include <fcntl.h>
#include "libft.h"
#include "builtins.h"
#include "minishell.h"
#include "parsing.h"
#include "prompt.h"

void	free_redir(void *redir_v)
{
	t_redirection	*redir;

	redir = redir_v;
	free(redir->file_name);
}

void	free_token(void *token_v)
{
	t_token	*token;

	token = token_v;
	free(token->argv);
	ft_lstclear(&(token->redirection), free_redir);
}

/*void	print_cmd(t_cmd cmd)
{
	ft_putendl_fd(cmd.cmd, 1);	
	while (*(cmd.argv))
	{
		ft_putendl_fd(*(cmd.argv), 1);
		(cmd.argv)++;
	}
}*/

void	print_pipe(t_list *pipe)
{
	t_list	*tmp;

	tmp = pipe;
	while (tmp)
	{
		ft_putendl_fd(tmp->content, STDOUT_FILENO);
		tmp = tmp->next;
	}
}

void	sigggg(int code)
{
	(void)code;
	ft_putchar_fd('\n', STDOUT_FILENO);
	rl_on_new_line();
	rl_replace_line("", 0);
	rl_redisplay();
}

void	siggg_backslash(int code)
{
	(void)code;
	rl_replace_line("", 0);
	rl_redisplay();
}

void	siggg_d(int code)
{
	(void)code;
	exit(EXIT_SUCCESS);
}

int	main(int ac, char **av, char **env)
{
	char	*test;
	char	*prompt;
	char	**lll;
	t_list	*piped;
	t_env	env_l;
	t_list	*cmd_list;

	(void)ac;
	(void)av;
	get_program_name(av[0]);
	piped = NULL;
	if (env_init(env, &env_l))
		return (EXIT_FAILURE);
	//sigemptyset(&(sigset_t){ SIGQUIT });
	signal(SIGINT, &sigggg);
	signal(SIGQUIT, &siggg_backslash);
	//signal(SIGSEGV, &siggg_d);
	while (1)
	{
		prompt = get_prompt(env_l);
		test = readline(prompt);
		if (!test)
			exit(727);
		free(prompt);
		add_history(test);
		if (check_syntax(test))
			continue ;
		lll = ft_split(test, ' ');
		if (!*lll)
			continue ;
		if (check_quote(test))
			continue ;
		if (check_pipe(test))
			continue ;
		piped = tokenizer(test);
		if (check_redir(((t_token *)(piped->content))->redirection))
			continue ;
		cmd_list = get_cmd_list(piped);
		format_quotes(cmd_list);
		exec_split_cmd(cmd_list, &env_l);
		free(test);
		ft_lstclear(&piped, free_token);
		ft_free("a", &lll);
	}
	ft_free("a", &lll);
	return (0);
}
